﻿TUI.Core.create({
	name: "TUI.map.BaseMap",
    init: function(isInit) {
        this._mapPanel = "mapPanel";                       // 局部变量，存储地图DIV的ID号
        this._map = null;                                    // 地图对象
		this._carMarkers = new TUI.util.HashMap();         // 哈希表，存储所有车辆的标签
		this._carLines = [];                                 // 只保存一辆车的实时轨迹

        this._historyData = [];                             // 数组，存储所有轨迹数据，由于需要分析每个点，所以用数组存储
        this._historyMarkers = [];                          // 数组，存储因轨迹产生的标记
        this._playMarker = null;                            // 轨迹播放标记

        this._counter = 0;                                 // 一个记数变量，产生不重复的数值
        this._region = null;                               // 存储区域对象，用来绘制区域
        this._infoWin = null;                              // 信息窗口
	},
	methods: {

        // 创建地图工具栏控件
		createControl: function() {
            var self = this;
			var imgHtml = '<img id="{0}" class="controlIcon" src="../images/ctrltools16X16/alpha_button_{0}.gif\" title="{1}" />';  // 按钮图片对应的HTML
            var splitHtml = '<img class="split" src="../images/ctrltools16X16/grid-blue-split.gif" />';  // 分隔符对应的HTML
			var img1 = "zoomIn", img2 = "zoomOut", img3 = "panH", img4 = "ruler", img5 = "fullExtent1", img6="clean";

            // 生成HTML
            var htmls = [];
			htmls.push( lg.maptools + ":" );
			htmls.push( imgHtml.format(img1,lg.zoomin) );
            htmls.push( splitHtml );
			htmls.push( imgHtml.format( img2, lg.zoomout ) );
            htmls.push( splitHtml );
			htmls.push( imgHtml.format( img3, lg.pan ) );
            htmls.push( splitHtml );
			htmls.push( imgHtml.format( img4, lg.ruler ) );
            htmls.push( splitHtml );
			htmls.push( imgHtml.format( img5, lg.fullextent ) );
            htmls.push( splitHtml );
            htmls.push( imgHtml.format( img6, lg.clean ) );
            htmls.push( splitHtml );

            // 地图选择控件
            htmls.push("<select id='select_map_type'>");
            htmls.push("<option value='0'>"+lg.baidumap+"</option>");
            htmls.push("<option value='1'>"+lg.baidusatellite+"</option>");
            htmls.push("<option value='2'>"+lg.googlemap+"</option>");
            htmls.push("<option value='3'>"+lg.googlesatellite+"</option>");
            htmls.push("<option value='4'>"+lg.mapabcmap+"</option>");
            htmls.push("</select>");

            // 加入地图中
			var parentNode = TUI.Dom.$( this._mapPanel).parentNode;
			var divNode = TUI.Dom.createElement("div",{className: "mapControl"});
			parentNode.appendChild(divNode);
			TUI.Dom.html( divNode, htmls.join("") );

            // 添加地图控件事件
            TUI.Event.addEventListener( img1, "click", function( e ) {
                self.dragMax();
            })
            TUI.Event.addEventListener( img2, "click", function( e ) {
                self.dragMin();
            })
            TUI.Event.addEventListener( img3, "click", function( e ) {
                self.moveMap();
            })
            TUI.Event.addEventListener( img4, "click", function( e ) {
                self.range();
            })
            TUI.Event.addEventListener( img5, "click", function( e ) {
                self.fullMap();
            })
            TUI.Event.addEventListener( img6, "click", function( e ) {
                self.cleanAll();
            })

            // 选中当前地图，并添加地图选择事件
            var select_map_type =  TUI.Dom.$("select_map_type");
            select_map_type.style.width = lang == 'cn'? '80px':'100px';
            select_map_type.options[maptype].selected = true;
            TUI.Event.addEventListener( select_map_type, 'change', function(){
                self.setMapType(select_map_type.selectedIndex);
            });
		},

        // 设置地图类型
        setMapType: function(index){
            // 如果没变，啥也不动
            if( index == maptype )
                return;
            // 如果地图对象已加载，并且是同一种地图，则只调用一次接口就行了，无须重新加载
            if(this._map){
                if( index == 0 && maptype == 1){
                    this._map.setMapType(BMAP_NORMAL_MAP);
                    maptype = index;
                    return;
                }
                else if( index == 1 && maptype == 0){
                    this._map.setMapType(BMAP_HYBRID_MAP);
                    maptype = index;
                    return;
                }
                else if( index == 2 && maptype == 3){
                    this._map.setMapTypeId(google.maps.MapTypeId.ROADMAP);
                    maptype = index;
                    return;
                }
                else if( index == 3 && maptype == 2){
                    this._map.setMapTypeId(google.maps.MapTypeId.HYBRID);
                    maptype = index;
                    return;
                }
            }
            // 切换地图
            window.location = "map.html?maptype=" + index +"&lang=" + lang;
        },

        // 清除所有
        cleanAll: function(){
            this.cleanCar();
            this.cleanHistoryLine();   // 清除历史轨迹
            this.cleanGeo();           // 清除空间对象
        },

        // 移除车辆标记
        removeCar: function( carId ) {
            var carMarkers = this._carMarkers;
            if ( carMarkers.containsKey( carId ) )
                this._removeOverlay(carMarkers.remove( carId ));

            // 如果轨迹是当前车辆的，则清空所有轨迹
            var carLines = this._carLines;
            if( carLines.carId == carId ){
                for( var i = 0; i < carLines.length; i ++ ){
                    this._removeOverlay(carLines[i]);
                }
                carLines.length  = 0 ;
                carLines.carId = null;
            }
        },

        // 清除所有车辆
        cleanCar: function() {
            var keys = this._carMarkers.getKeys();
            for( var i = 0; i < keys.length; i ++ ){
                this.removeCar(keys[i]);
            }
        },

        // 清空轨迹图层
        cleanHistoryLine: function() {
            var self = this;
            var overlays = this._historyMarkers;
            for( var i = 0; i < overlays.length;  i ++ ){
                self._removeOverlay(overlays[i]);
            }
            // 清除历史轨迹
            this._historyMarkers = [];
            this._historyData = null;
            // 如果创建了播放标记
            if( this._playMarker){
                self._removeOverlay(this._playMarker);
                this._playMarker = null;
            }
        },

		//////////////////////////////Tip窗口对应HTML代码/////////////////////////////////////
		// 获取车辆简要信息提示
		getSimpleCarTip: function( pos ) {
            var html = [];
            html.push("<div class=\"simpleCarTip\">");
            html.push("<div><span style='color:#11aabf;'>"+lg.gpstime+":</span>{0}<div/>");
            html.push("<div><span style='color:#11aabf;'>"+lg.pointed+":</span>{1}<div/>");
            html.push("<div><span style='color:#11aabf;'>"+lg.speed+":</span>{2}<div/>");
            html.push("<div><span style='color:#11aabf;'>"+lg.status+":</span>{3}<div/>");
            html.push("</div>");
			return html.join("").format(
                pos.gpstime,
                this._getPointed(pos.pointed),
                this._getSpeed(pos.speed),
                this._getStatus(pos.scode)
			);
		},
		
		// 获取车辆详细信息提示
		getDetailCarTip: function( carNO, pos ) {
            var html = [];
            html.push("<div class=\"detailCarTip\">");
            html.push("<div style='font-size:14px;font-weight:bold'>{0}</div>");   // 标题
            html.push("<div style='border-bottom:1px solid #ff0000;'></div>");     // 红线
            html.push("<table cellpadding=\"0\" cellspacing=\"0\"");
            html.push("<tr>");
            html.push("<td><label>"+lg.gpstime+":</label><span>{1}</span></td>");
            html.push("<td><label>"+lg.speed+":</label><span>{2}</span></td>");
            html.push("</tr>");
            html.push("<tr>");
            html.push("<td><label>"+lg.pointed+":</label><span>{3}</span></td>");
            html.push("<td><label>"+lg.status+":</label><span>{4}</span></td>");
            html.push("</tr>");
            html.push("<tr>");
            html.push("<td><label>"+lg.lo+":</label><span>{5}</span></td>");
            html.push("<td><label>"+lg.alarm+":</label><span>{6}</span></td>");
            html.push("</tr>");
            html.push("<tr>");
            html.push("<td><label>"+lg.la+":</label><span>{7}</span></td>");
            if( pos.stopTime > 0)   // 停车时长大于0才显示
                html.push("<td><label>"+lg.stoptime+":</label><span>{8}</span></td>");
            html.push("</tr>");
            html.push("<tr>");
            html.push("<td colspan='2'><label>"+lg.address+":</label><span>"+lg.searching +"</span></td>");
            html.push("</tr>");
            html.push("</table>");
            html.push("</div>");
			return html.join("").format(
                carNO,
                pos.gpstime,
                this._getSpeed(pos.speed),
                this._getPointed(pos.pointed),
                this._getStatus(pos.scode),
                this._getLo(pos.lo),
                this._getAlarm(pos.acode),
                this._getLa(pos.la),
                this._getStopTime(pos.stopTime)
			);
		},

        // 根据经纬度生成位置描述
        getAddress : function( lng, lat, callid ) {
            // 向服务器请求位置描述
            try{
                this._callid = callid;
                window.external.getAddress(lng,lat,callid);
            }
            catch(e){}
        },

        // 由外部调用，实现地址转换
        getAddressCallback:function(callid, address){
            if( this._callid == callid ){
			var ct = this._infoWin.getContent().replace(lg.searching, address);
                this._infoWin.setContent(ct);
            }
        },

        // 刷新线的属性，基本都有getPath函数，如果遇到不一致的，再考虑重载吧
        _refreshPolyline:function(poly, opt ){
            if(poly && opt.add ){            // 如果刷新只是添加点
                var path = poly.getPath();   // 获取路径
                path.push( opt.add );        // 添加点
                poly.setPath( path );        // 更新路径
            }
        },

        // 显示车辆
        showCar:function(car, fit){
	        if( !car || !car.pos )
		        return;
            var carMarkers = this._carMarkers;
            // 更新车辆图标
            var opt = {
                lngLat:this._newLngLat( car.pos.lng, car.pos.lat ),  // 位置
                icon:"../images/carIcon/" + this._getCarIcon(car.carIcon, car.pos.direction),           // 图标
                size:{x:32,y:32},                                       // 图标大小
                label:car.carNO + "," + this._getSpeed(car.pos.speed),           // 标题
                offset:{x:32,y:6}                                      // 标题偏移
            };
            var marker = carMarkers.get(car.carId);
            if( marker == null ){   // 如果原来不存在该图标
                marker = this._newMarker(opt);
                this._addOverlay( marker );      // 添加到地图上
                this._addInfoWindow( marker, car, car.pos );   // 添加信息窗口
                carMarkers.put( car.carId, marker );           // 加入到车辆标记列表
            }
            else{
                this._refreshMarker(marker, opt);
                this._addInfoWindow( marker, car, car.pos );   // 重新添加信息窗口
                if( this._infoWin && this._infoWin.carId == car.carId){  // 如果当前信息窗口正在显示是本车，则更新信息窗口
                    this._refreshInfoWindow(marker,car, car.pos);
                }
            }
            // 如果需要聚焦到车辆上
            if( fit ){
                this.setCenter(car.pos.lng, car.pos.lat, 16);
            }
        },

        // 绘制轨迹
        // 轨迹中只带有carId，不能显示车牌号，显然不行
        drawHistoryLine: function( car, historys ,stopMark ) {
            // 如果已经有轨迹，先清除
            if ( this._historyMarkers.length > 0 )
                this.cleanHistoryLine();
            // 保存下轨迹数据先
            this._historyData = historys;
            // 有起始点，则先绘制起始点
            if( historys.length > 0){
                var first = historys[0];
                var opt = {
                    lngLat:this._newLngLat( first.lng, first.lat ),  // 位置
                    icon:"../images/carIcon/startpos" + (lang=='cn'?'':'_'+lang) +  ".gif",  // 图标
                    size:{x:41,y:26}                                       // 图标大小
                };
                var firstMarker = this._newMarker( opt );
                this._addOverlay(firstMarker);
                this._addInfoWindow(firstMarker, car, first);
                this._historyMarkers.push( firstMarker );
            }
            // 有超过1个点
            if (historys.length > 1)
            {
                // 生成结束点
                var last  = historys[ historys.length - 1 ];
                var opt = {
                    lngLat:this._newLngLat( last.lng, last.lat ),  // 位置
                    icon:"../images/carIcon/stoppos" + (lang == 'cn'?'':'_'+lang) +  ".gif",  // 图标
                    size:{x:41,y:27}                    // 图标大小
                };
                var lastMarker = this._newMarker( opt );
                this._addOverlay(lastMarker);
                this._addInfoWindow(lastMarker, car, last);
                this._historyMarkers.push( lastMarker );

                // 遍历所有轨迹，生成需要的标签
                var lngLats = [];
                var color = null;
                for( var i = 0; i < historys.length; i ++ ){
                    var item = historys[i];
                    lngLats.push( this._newLngLat(item.lng, item.lat) );  // 添加到数组构成线
                    var col = this._getColorBySpeed(item.speed);  // 根据速度取出当前颜色
                    if(color){
                        if( col != color ){   // 如果颜色变化
                            var polyline = this._newPolyline( {lngLat:lngLats, color:color} );     // 绘制轨迹
                            this._addOverlay(polyline);
                            this._historyMarkers.push( polyline );
                            lngLats = [];  // 清空轨迹点
                            lngLats.push( this._newLngLat(item.lng, item.lat) );  // 添加当前点到数组
                            color = col;                                           // 更新当前颜色
                        }
                    }else{  // 如果没有初始颜色，则取个起点颜色
                        color = col;
                    }
                    // 如果报警是以a开头，表示有告警
                    if( item.acode && item.acode.length > 2 ){
                        // 画报警点
                        var opt = {
                            lngLat:this._newLngLat( item.lng, item.lat ),  // 位置
                            icon:"../images/carIcon/alarm.gif",         // 图标
                            size:{x:16,y:24}                                // 图标大小
                        };
                        var alarmMarker = this._newMarker(opt);
                        this._addOverlay(alarmMarker);
                        this._addInfoWindow(alarmMarker, car, item);
                        this._historyMarkers.push( alarmMarker );
                    }
                    // 如果有停车时长，则需要显示停车点
                    if( stopMark && item.stopTime > 0 ){
                        // 画停车点
                        var opt = {
                            lngLat:this._newLngLat( item.lng, item.lat ),    // 位置
                            icon:"../images/carIcon/m_stop.gif",         // 图标
                            size:{x:12,y:20},                                 // 图标大小
                            label:lg.stopcar + ':' + this._getStopTime(item.stopTime),
                            offset:{x:10,y:15}
                        };
                        var stopMarker = this._newMarker(opt);
                        this._addOverlay(stopMarker);
                        this._addInfoWindow(stopMarker, car, item);
                        this._historyMarkers.push( stopMarker );
                    }
                }
                // 绘制轨迹
                if( lngLats.length >= 2 ){
                    var polyline = this._newPolyline({lngLat:lngLats,color:color});
                    this._addOverlay(polyline);
                    this._historyMarkers.push( polyline );
                }
                // 调整窗口大小
                this._setFitview( this._historyMarkers );
            }
        },

        // 移动到index帧
        moveTo: function( car, index , interval) {
            var self = this;
            var historys = this._historyData;
            if( historys == null || historys.length == 0 || index >= historys.length )
                return;
            // 算出当前位置对应的数据
            var history = historys[index]
            // 播放标记
            var playMarker = this._playMarker;
            // GPS中的方向字段经常没有，通过小车路线来区分方向
            var dir = history.direction;
            if( index > 0){   // 如果有前一个点，则用这两个点构造方向
                var last = historys[index - 1];
                dir = this._getAngle(last.lng, last.lat, history.lng, history.lat);
            }
            // 构造参数
            var opt = {
                lngLat: this._newLngLat( history.lng, history.lat ),  // 位置
                icon:"../images/carIcon/" + this._getCarIcon( car.carIcon ,dir),         // 图标
                size:{x:32,y:32},                                 // 图标大小
                label: this.getSimpleCarTip( history ),
                offset:{x:32,y:6}
            };
            // 如果还没创建播放点，则创建
            if( playMarker == null ){
                playMarker = this._newMarker( opt );
                this._addOverlay( playMarker );
                this._playMarker = playMarker;
            }
            // 已经创建则只更新
            else{
                if( self.timer )  // 重新刷新点时，一定要停止动画
                    clearTimeout(self.timer);
                // 如果可以实现动画
                if( interval > 0 && index > 0 ){
                    var last = historys[index -1];
                    // 改变一下图标，位置用上一次的位置，慢慢移动过去
                    opt.lngLat = this._newLngLat(last.lng,last.lat);
                    this._refreshMarker(playMarker, opt);

                    var dx = (history.lng - last.lng)/10.0;
                    var dy = (history.lat - last.lat)/10.0;
                    var i = 10;
                    var doMove = function(){
                        var lng = history.lng - dx * i;
                        var lat = history.lat - dy * i;
                        playMarker.setPosition(self._newLngLat(lng, lat));  // 移动标签
                        if( i-- > 0 ){
                            self.timer = setTimeout(doMove, Math.round(interval) );
                        }
                    }
                    doMove();  // 执行动画效果
                }else{   // 不使用动画效果
                    this._refreshMarker(playMarker, opt);
                }
            }
            
            // 如果当前信息窗口正在显示,则关闭
            if( this._infoWin ){
                this._infoWin.close();
				this._infoWin = null;
            }

			// 添加信息窗口
            this._addInfoWindow( playMarker,car, history );

            // 调整窗口位置
            this.lockWindowByLngLat( opt.lngLat );
        },

        // 获取角度
        _getAngle :function (x1, y1, x2, y2) {
            var dx = x2 - x1;
            var dy = y2 - y1;
            var rad = Math.atan( Math.abs(dx/dy) );
            var angle =  rad * 180.0 / Math.PI;
            if( dx > 0 && dy <= 0 )  // 二象限
                return 180 - angle;
            else if ( dx <=0 && dy < 0 )  // 三象限
                return 180 + angle;
            else if ( dx < 0 && dy >= 0 )  // 四象限
                return 360 - angle;
            else
                return angle;
        },
        // 根据速度生成颜色
        _getColorBySpeed: function(speed){
            if(speed >= 100)
                return "red";
            else if( speed >= 60 && speed < 100 )
                return "#FF00FF";
            else
                return "green";
        },
        // 获取车辆图片
        _getCarIcon: function(carIcon, dir){
            if( isNaN(dir) || dir < 0 || dir > 360 )
                dir = 0;
            switch(carIcon){
                case 1:
                    return 'moto/0.png';
                case 2:
                    dir = Math.round( dir / 30 ) % 12;
                    return 'car/'+dir+'.png';
                case 3:
                    dir = Math.round( dir / 90 ) % 4;
                    return 'truck/' + dir + '.png';
                case 4:
                    dir = Math.round( dir / 90 ) % 4;
                    return 'police/' + dir + '.gif';
                case 5:
                    dir = Math.round( dir / 45 ) % 8;
                    return 'dir/' + dir + '.png';
                case 8:
                    return 'bulldozer/0.png';
                case 7:
                    return 'excavator/0.png';
                default:
                    return 'drop/0.gif';
            }
        },
        // 解析状态字符串
        _getStatus: function(scode){
            var status = [];
            if( scode ){
                if( scode.charAt(0)=='s'){
                    var ss = scode.split(':');
                    for(var i = 1 ; i < ss.length; i++){
                        status.push( lg.statusarray[ss[i]] );
                    }
                } else {
                    status.push(scode);
                }
            }
            return status.join(",");
        },
        // 解析车辆报警
        _getAlarm : function(acode){
            var alarm = [];
            if( acode ){
                if( acode.charAt(0) == 'a' ){
                    var ss= acode.split(':');
                    for(var i = 1; i< ss.length; i++){
                        var aa = ss[i].split(',');
                        if( aa.length > 1 )  // 如果带有参数
                            alarm.push(lg.alarmarray[aa[0]] + "[" + aa[1] + "]");
                        else                  // 没有参数
                            alarm.push(lg.alarmarray[aa[0]]);
                    }
                }
                else{
                    alarm.push(acode);
                }
            }
            return alarm.join(",");
        },
        // 获取定位方式
        _getPointed : function(pointed) {
            return lg.pointedarray[ pointed % lg.pointedarray.length ];
        },
        // 获取方向
        _getDirection : function(dir) {
            return lg.directionarray[ Math.round(dir/45) % lg.directionarray.length ];
        },
        // 获取速度
        _getSpeed : function(speed) {
            return speed + " km/h";
        },

        // 获取停车时长
        _getStopTime:function(stopTime){
            if( stopTime > 0){
                return TUI.Core.parseSeconds(stopTime);
            }else{
                return "";
            }
        },
        // 格式化经度
        _getLo: function(lo){
            return lo.toFixed(6) + "°" + ( lo > 0 ? lg.directarray[0] : lg.directarray[2] );
        },
        // 格式化纬度
        _getLa:function(la){
            return la.toFixed(6) + "°" + ( la > 0 ? lg.directarray[3] : lg.directarray[1] );
        },

        /////////////////////////////////////按全管理事件，防止未释放///////////////////////
        // 安全注册事件
        _safeBind: function( elem, event, fun){
            this.events = this.events ? this.events : [];   // 有一个数组来存储所有事件
            this._cleanBind(elem, event);
            this._bind(elem, event, fun);
            this.events.push( {elem:elem,event:event,fun:fun});
        },

        // 根据条件返回对象  参数允许为空
        _getBind:function(elem, event){
            var ret = [];
            if( this.events ){
                for(var i = 0; i < this.events.length; i ++ ){
                    var cur = this.events[i];
                    if ( (!elem || cur.elem == elem ) && (!event || cur.event == event) )
                        ret.push(cur);
                }
            }
            return ret;
        },

        // 释放绑定事件 参数允许为空，为空则选择所有
        _cleanBind:function(elem, event){
            var ret = this._getBind( elem , event );
            for( var i = 0; i < ret.length; i ++ ){
                var cur = ret[i];
                this._unbind( cur.elem, cur.event, cur.fun );
                this.events.remove(cur);
            }
        },
        /////////////////////////////////////空间对象相关///////////////////////////////////
        // json转成经纬度数组
        _parsePoints : function (points){
            var res = [];
            for( var i = 0; i < points.length; i ++ ){
                res.push( this._newLngLat(points[i].lng, points[i].lat) );
            }
            return res;
        },

        // 清空所有空间对象
        cleanGeo: function(){
            if( this._geo ){
                this._removeOverlay(this._geo);
                this._geo = null;
            }
        },
        // 获取对象坐标
        endGeo:function(){
            if( this._geo ) {
                var points = "";
                var path = [];
                if( this._geo.getPath ){
                    path = this._geo.getPath();
                }
                else{
                    path.push(this._geo.getPosition());
                }
                // 将所有点连成一个字符串
                for(var i = 0; i< path.length ;i++){
                    if( i != 0 )
                        points += ';';
                    points += path[i].lng.toFixed(6) + ',' + path[i].lat.toFixed(6);
                }
                // 传递坐标给上层
                try{
                    window.external.onEndGeo(points);
                }
                catch(e){}
                
		        this._cleanBind(this._geo);  // 清空绑定的事件
                this._cleanBind(this._map);
            }
        }
    }
});